/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { C as ColorPalette } from './colors.js';
import { e as extractColor } from './utils.js';
import { c as colors } from './colors2.js';

class ColorHelper {
  /**
   * @param color - color string in format of `colorName-tone` or `colorName` (default tone is 600)
   * @return `{ tone: number, colorName: string }`
   * @description
   * Parse color string to get color name and tone
   * @example
   * parseColor("red-500") // { tone: 500, colorName: "red" }
   * parseColor("red") // { tone: 600, colorName: "red" }
   * parseColor(null) // { tone: null, colorName: null }
   */
  static parseColor(color, defaultTone = 600) {
    if (!color)
      return { tone: null, colorName: null };
    const [colorName, tone] = color.split("-");
    const parsedTone = tone ? parseInt(tone) : defaultTone;
    return {
      tone: parsedTone,
      colorName,
    };
  }
}
ColorHelper.isHex = (str) => /(^#[0-9A-F]{6}$)|(^#[0-9A-F]{3}$)/i.test(str);
ColorHelper.isRgb = (str) => /^rgb\((\d{1,3}),\s*(\d{1,3}),\s*(\d{1,3})\)$/.test(str);
ColorHelper.isRgba = (str) => /^rgba\((\d{1,3}),\s*(\d{1,3}),\s*(\d{1,3}),\s*(\d{1,3})\)$/.test(str);
ColorHelper.isHsl = (str) => /^hsl\((\d{1,3}),\s*(\d{1,3}%),\s*(\d{1,3}%)\)$/.test(str);
ColorHelper.isHsla = (str) => /^hsla\((\d{1,3}),\s*(\d{1,3}%),\s*(\d{1,3}%),\s*(\d{1,3})\)$/.test(str);
ColorHelper.isVariable = (str) => /^var\((.*)\)$/.test(str);
ColorHelper.extractVariable = (str) => str.match(/^var\((.*)\)$/)[1];
ColorHelper.convertColorToHex = (color) => {
  if (color) {
    if (ColorHelper.isVariable(color)) {
      const extractColor = ColorHelper.extractVariable(color);
      return getComputedStyle(document.documentElement).getPropertyValue(extractColor);
    }
    if (ColorHelper.isHex(color)) {
      return color;
    }
    if (ColorHelper.isRgb(color)) {
      const rgb = color.match(/^rgb\((\d{1,3}),\s*(\d{1,3}),\s*(\d{1,3})\)$/);
      return `#${(0x1000000 + parseInt(rgb[1]) * 0x10000 + parseInt(rgb[2]) * 0x100 + parseInt(rgb[3])).toString(16).substring(1)}`;
    }
    if (extractColor(ColorPalette, color)) {
      return extractColor(ColorPalette, color);
    }
  }
};
ColorHelper.reverseColor = (color) => {
  let hexColor = ColorHelper.convertColorToHex(color).trim();
  if (hexColor) {
    if (hexColor.length === 4) {
      hexColor = `#${hexColor[1]}${hexColor[1]}${hexColor[2]}${hexColor[2]}${hexColor[3]}${hexColor[3]}`;
    }
    const r = parseInt(hexColor.substring(1, 3), 16);
    const g = parseInt(hexColor.substring(3, 5), 16);
    const b = parseInt(hexColor.substring(5, 7), 16);
    const yiq = (r * 299 + g * 587 + b * 114) / 1000;
    return yiq >= 200 ? '#334155' : '#fff';
  }
};
ColorHelper.dsColor = (color) => 'var(--bcm-new-ds-color-' + color + ')';
ColorHelper.getTwColorToHex = (colorName) => ColorHelper.getTwColor(colorName) || colors['blue']['500'];
ColorHelper.getTwColor = (colorName, checkTw = false) => {
  if (colors[colorName]) {
    return colors[colorName];
  }
  const splitted = colorName.split('-');
  const grade = splitted[splitted.length - 1];
  const swatch = splitted.slice(0, splitted.length - 1).join('-');
  if (colors[swatch] && colors[swatch][grade]) {
    return checkTw ? 1 : colors[swatch][grade];
  }
  else if (colors[swatch]) {
    return checkTw ? 2 : colors[swatch];
  }
  else {
    console.warn('Color not found in tailwind colors: ' + colorName);
    return 0;
  }
};
// slate => slate-500
// slate-700 => slate-700
ColorHelper.twColor = (colorName, tone = '500') => {
  const newColor = colorName && colorName.indexOf('-') === -1 ? `${colorName}-${tone}` : colorName;
  const getTw = ColorHelper.getTwColor(newColor, true);
  if (getTw === 1) {
    return newColor;
  }
  if (getTw === 2) {
    return colorName;
  }
  return 'blue-500';
};
ColorHelper.twColorReverse = (color) => ColorHelper.reverseColor(ColorHelper.getTwColorToHex(color));
/**
 * @param object
 * @return string
 * @description Convert style object to string
 */
ColorHelper.generateStyleString = (obj) => Object.keys(obj).reduce((acc, key) => { return `${acc}${key}:${obj[key]};`; }, "");

export { ColorHelper as C };
