/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { a as getTransform } from './utils-57652744.js';

/**
 * Enums
 */
var DragDirections;
(function (DragDirections) {
  DragDirections["x"] = "x";
  DragDirections["y"] = "y";
  DragDirections["both"] = "both";
})(DragDirections || (DragDirections = {}));
/**
 * @Class BcmDragger
 * @desc -
 */
class ElementDragger {
  constructor({ target, trigger, limit, center, direction = DragDirections.both, snapOnMouseDown = false, useTransform = false }) {
    this.drag = {
      x: 0,
      y: 0
    };
    this.offset = {
      left: 0,
      top: 0
    };
    this.events = {
      end: [],
      drag: []
    };
    this.active = false;
    this.anim = false;
    this.busy = false;
    this.target = target;
    this.trigger = trigger || target;
    this.direction = direction;
    this.limit = limit;
    this.useTransform = useTransform;
    this.snapOnMouseDown = snapOnMouseDown;
    this.center = center;
    this.container = this.findRelativeContainer();
    // Get trigger offset initially
    // #
    this.offset = {
      top: this.useTransform
        ? getTransform(target).y
        : target.offsetTop,
      left: this.useTransform
        ? getTransform(target).x
        : target.offsetLeft
    };
    this.attachEvents();
  }
  /**
   * @desc
   */
  attachEvents() {
    /**
     * Mouse Event: move -> [window]
     */
    window.addEventListener('mousemove', (_event) => this.windowMove(_event));
    /**
     * Mouse Event: move -> [window]
     */
    window.addEventListener('mouseup', (_event) => this.windowUp());
    /**
     * Mouse Event: down -> [BcmDragger.trigger]
     */
    this.trigger.addEventListener('mousedown', (_event) => this.triggerDown(_event));
  }
  /**
   * @desc
   * @param element
   * @param property
   */
  computedStyle(element, property) {
    return window.getComputedStyle(element, null).getPropertyValue(property);
  }
  /**
   * @desc
   * -
   */
  findRelativeContainer() {
    const { target } = this;
    const elementNode = Node.ELEMENT_NODE;
    let currentParent = target.parentNode;
    while (currentParent && currentParent.nodeType === elementNode) {
      const match = ['absolute', 'relative'].includes(this.computedStyle(currentParent, 'position'));
      if (match)
        return currentParent;
      currentParent = currentParent.parentNode;
    }
    return null;
  }
  /**
   * @desc
   * @param event
   */
  windowMove(event) {
    if (!this.active)
      return;
    const { limit } = this;
    const { center: { x: centerX, y: centerY } = { x: 0, y: 0 }, } = this;
    let x = event.pageX - this.drag.x - centerX;
    let y = event.pageY - this.drag.y - centerY;
    this.busy = true;
    // Check limit boundaries
    // #
    const [xMax, xMin, yMax, yMin] = [
      limit && limit.x && limit.x.max - centerX,
      limit && limit.x && limit.x.min - centerX,
      limit && limit.y && limit.y.max - centerY,
      limit && limit.y && limit.y.min - centerY
    ];
    if ((xMax || xMax === 0) && x > xMax) {
      x = xMax;
    }
    if ((xMin || xMin === 0) && x < xMin) {
      x = xMin;
    }
    if ((yMax || yMax === 0) && y > yMax) {
      y = yMax;
    }
    if ((yMin || yMin === 0) && y < yMin) {
      y = yMin;
    }
    this.offset = {
      top: y,
      left: x
    };
    this.removeTransition();
    this.move();
  }
  /**
   * @desc
   */
  windowUp() {
    if (!this.active)
      return;
    this.triggerEvent('end', this.getCurrentPosition());
    this.animTimeout = setTimeout(() => {
      clearTimeout(this.animTimeout);
      this.busy = false;
    }, 210);
    this.active = false;
  }
  /**
   * @desc
   * @param event
   */
  triggerDown(event) {
    this.active = true;
    if (this.snapOnMouseDown) {
      this.offset.left = event.offsetX;
      this.offset.top = event.offsetY;
      this.move();
    }
    this.drag.x = event.pageX - this.offset.left;
    this.drag.y = event.pageY - this.offset.top;
  }
  /**
   * @desc
   */
  move(triggerDrag = true) {
    const { direction, useTransform, target, offset: { top, left } } = this;
    if (direction === DragDirections.both) {
      useTransform
        ? target.style.transform = `translate(${left}px, ${top}px)`
        : (target.style.top = `${top}px`) && (target.style.left = `${left}px`);
    }
    else if (direction === DragDirections.x) {
      useTransform
        ? target.style.transform = `translateX(${left}px)`
        : target.style.left = `${left}px`;
    }
    else if (direction === DragDirections.y) {
      useTransform
        ? target.style.transform = `translateY(${top}px)`
        : target.style.top = `${top}px`;
    }
    if (triggerDrag)
      this.triggerEvent('drag', this.getCurrentPosition());
  }
  /**
   * @desc
   * -
   */
  calculateOffset() {
    const { target, container } = this;
    let { left, top } = target.style;
    if (!container) {
      return { left, top };
    }
    const [rectTarget, rectContainer] = [
      target.getBoundingClientRect(),
      container.getBoundingClientRect()
    ];
    left = rectTarget.left - rectContainer.left,
      top = rectTarget.top - rectContainer.top;
    return { left, top };
  }
  /**
   * @desc
   */
  getCurrentPosition() {
    const { target } = this;
    const { center: { x: centerX, y: centerY } = { x: 0, y: 0 } } = this;
    let { left: x, top: y } = this.calculateOffset();
    // 'offsetLeft' and 'offsetTop' properties are giving 'long integer' values
    // in this case, this is not useful, we are using subpixel values. 
    return {
      x: this.useTransform
        ? getTransform(target).x + centerX
        : /* this.target.offsetLeft */ parseFloat(x) + centerX,
      y: this.useTransform
        ? getTransform(target).y + centerY
        : /* this.target.offsetTop */ parseFloat(y) + centerY
    };
  }
  /**
   * @desc
   * @param x
   * @param y
   */
  setPosition({ x, y, anim }) {
    const { center: { x: centerX, y: centerY } = { x: 0, y: 0 }, } = this;
    if (x || x === 0) {
      this.offset.left = x - centerX;
    }
    if (y || y === 0) {
      this.offset.top = y - centerY;
    }
    anim && this.applyTransition();
    this.move(false);
  }
  /**
   * @desc
   */
  applyTransition() {
    this.anim = true;
    this.target.style.transition = 'transform 0.2s ease';
  }
  /**
   * @desc
   */
  removeTransition() {
    this.anim = false;
    this.target.style.transition = '';
  }
  /**
   * @desc
   * @returns
   */
  getOffset() {
    return this.offset;
  }
  /**
   * @desc
   * @param event
   * @param fn
   */
  on(event, fn) {
    this.events[event].push(fn);
  }
  /**
   * @desc
   * @param event
   */
  triggerEvent(event, ...params) {
    this.events[event].forEach((fn) => fn(params[0]));
  }
}

export { DragDirections as D, ElementDragger as E };
